@@ -9,7 +9,8 @@ namespace image {
 C10_EXPORT torch::Tensor decode_png(
     const torch::Tensor& data,
     ImageReadMode mode = IMAGE_READ_MODE_UNCHANGED,
-    bool allow_16_bits = false);
+    bool allow_16_bits = false,
+    bool apply_exif_orientation = false);
 
 } // namespace image
 } // namespace vision
